package ru.dbotthepony.kstarbound.defs.image

import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.util.PathStack
import ru.dbotthepony.kstarbound.util.SBPattern

/**
 * Хранит данные (пару) вида "/example/image.png:sprite.name"
 */
data class SpriteReference(
	val image: String,
	val atlas: AtlasConfiguration,
	val sprite: SBPattern
) {
	val stateless by lazy { resolve { null } }

	fun resolve(with: (String) -> String?): AtlasConfiguration.Sprite {
		val resolved = sprite.resolve(with) ?: return atlas.any()
		return atlas[resolved] ?: atlas.any()
	}

	class Adapter(private val remapper: PathStack, private val atlasRegistry: () -> AtlasConfiguration.Registry) : TypeAdapter<SpriteReference>() {
		override fun write(out: JsonWriter, value: SpriteReference?) {
			if (value == null)
				out.nullValue()
			else
				out.value(value.image + ":" + value.sprite.raw)
		}

		override fun read(`in`: JsonReader): SpriteReference? {
			val value = `in`.nextString() ?: return null

			// TODO: если нам надо принудительно удалить спрайт с вышестоящей ступени, как это сделать?
			// а ещё, зачем вы это сделали.
			if (value == "")
				return null

			return parse(remapper.remap(value))
		}

		fun parse(input: String): SpriteReference {
			val grid = atlasRegistry.invoke().get(input.substringBefore(':'))

			return when (input.count { it == ':' }) {
				0 -> SpriteReference(input, grid, SBPattern.raw(grid.any().name))
				1 -> SpriteReference(input.substringBefore(':'), grid, SBPattern.of(input.substringAfter(':')))
				else -> throw IllegalArgumentException("Invalid sprite reference: $input")
			}
		}
	}
}
